/*!
 * \file qftFile.qt.hpp
 * \brief This file contains the member functions of the class QftFile which helps to handle file sequences
 *
 * \author  Dipl. Inf. Manfred Reimann
 *
 * $Revision$
 *
 * <dl class="rcs">
 *    <dt><b>Product Name:</b></dt><dd>FTGS Dacff</dd>
 * </dl>
 *
 * $Date$
 *
 * <dl class="rcs">
 *    <dt><b>Classification:</b></dt><dd>UNCLASSIFIED</dd>
 * </dl>
 *
 * <dl class="rcs">
 *    <dt><b>Copyright:</b></dt><dd>&copy; EADS 2010</dd>
 * </dl>
 *
 * <dl class="rcs">
 *    <dt><b>Description:</b></dt><dd>File Handling of qft files (sequences of files)</dd>
 * </dl>
 *
 * \todo documentation missing and version number
 *
 *************************************************************************
 * <dl class="rcs"><dt><b>History:</b></dt><dd>
 * <pre>
 * $Log$
 * </pre> 
 * 21.09.07 MR reincarnation<br>
 * 12.03.07 MR initiation<br>
 * </dd></dl>
 *************************************************************************
*/

#include "qftFile.qt.hpp"
#include "QDebug"
#include "QDir"
#include "QRegExp"
#include "QStringList"

/*!
 * \brief Set the permissions to the file sequence
 *
 * Set or changes the file permissions for the file sequence.
 *
 * \todo Does not work at the moment, fix problems here.
 *
 * \param[in] 	permissions New file permissions to set
 * \return bool: 	true if no error occurs, false if an error occurs
 *			(in this case abort the setting)<br>
 *			If the change operation fails for one file, the process is aborted without resetting the
 *			permissions of already changed files.
 */
bool QftFile::setPermissions( QFile::Permissions permissions){

    foreach( QftFileInfo info, m_infoVector ){
        bool r = QFile::setPermissions( info.filePath(), permissions );

        if( !r ) return false;
    }
    return true;
}



/*!
 * \brief Default constructor
 *
 * Initialise the qft file class:<br>
 * Set the open mode to not opened and set a maximal file size.
 */
QftFile::QftFile( void ) :
    m_openMode( QIODevice::NotOpen ){
    dataSizeLimit = 1024 * 1024 * 1024;
}

/*!
 * \brief Alternative constructor
 *
 * Initialise a qft file class and additional set the name of the qft file sequence.
 *
 */
QftFile::QftFile( const QString &name //! \param[in] name Name of the qft file sequence
) :
    m_openMode( QIODevice::NotOpen ){
    dataSizeLimit = 1024 * 1024 * 1024;

    m_init( name );
}

/*!
 * \brief Destructor closes file
 *
 * Close the currently opened qft file object (only one file of the sequence is opened at the same time).
 */
QftFile::~QftFile(){

    close();
}

bool QftFile::open( const char *mode){
//    qDebug() << __PRETTY_FUNCTION__;

    switch( *mode++ ){
        case 'w':
            return open((( *mode == '+' ) ? QIODevice::ReadWrite
                                          : QIODevice::WriteOnly ));
        case 'r':
            return open((( *mode == '+' ) ? QIODevice::ReadWrite
                                          : QIODevice::ReadOnly ));
        case 'a':
            return open( QIODevice::Append );
    }
    return false;
}

/*! \brief Open the file sequence (mode parameter QIODevice)
 *
 * Open the file sequence and set(read / write / append) process position to the beginning.<br>
 * Parameter mode is already an element of QIODevice.<br>
 * Additional some validation checks are done:<br>
 * - Filename not empty<br>
 * - File not already opened (otherwise print a warning message)<br>
 *
 * Only ReadOnly and WriteOnly modes are currently supported.<br>
 * The info vector position and local file position are set to 0 to start at position 0.<br>
 * In WriteOnly mode the file sequence files get cleared (deleted) first.<br>
 * In WriteMode also the variable m_rest is set to maximum data size.<br>
 * In Read mode the variable m_rest is set to the size of the first QftFileInfo object in m_infoVector. <br>
 *
 \return bool: Return false if the file could not be opened otherwise true.
 */
bool QftFile::open( QIODevice::OpenMode mode){
//    qDebug() << __PRETTY_FUNCTION__;

    if( m_fileName.isEmpty() ) return false;
//    qDebug() << __PRETTY_FUNCTION__;

    if( m_openMode != QIODevice::NotOpen ){
        qWarning("QftFile::open: File already open");
        return true;
    }

    switch( mode ){

        case QIODevice::WriteOnly:
            foreach( QftFileInfo info, m_infoVector )
                QFile::remove( info.fileName() );

            m_infoVector.clear();
            m_infoVector.append( QftFileInfo( m_baseName.arg( 0, m_indexLength, 10, QChar( '0' ))));
            m_infoIndex = 0;
            m_filePosition = 0;
            m_rest = dataSizeLimit;
//            mode |= QIODevice::Unbuffered;
            break;

        case QIODevice::ReadWrite:
//            mode |= QIODevice::Unbuffered;
            break;

        case QIODevice::ReadOnly:
            m_infoIndex = 0;
            m_filePosition = 0;
            m_rest = m_infoVector[ 0 ].size();
//            mode |= QIODevice::Unbuffered;
            break;

        case QIODevice::Append:
            break;
    }
    m_openMode = mode; // | QIODevice::Unbuffered ;

//    qDebug() << m_infoVector[ m_infoIndex ].filePath();

    m_file.setFileName( m_infoVector[ m_infoIndex ].filePath() );

    if( !m_file.open( m_openMode ) ){
        qWarning("QftFile::open: File cannot be opened");
        return false;
    }
    emit sigStarted( m_infoIndex );
    return true;
}

void QftFile::close(){
//    qDebug() << __PRETTY_FUNCTION__ << "IN";
    if( m_openMode != QIODevice::NotOpen )
        m_openMode = QIODevice::NotOpen;

//    qDebug() << __PRETTY_FUNCTION__ << "STEP";

    if( m_file.openMode() != QIODevice::NotOpen )
        m_file.close();
//    qDebug() << __PRETTY_FUNCTION__ << "OUT";

}

/*!
 * \brief Read file sequence
 *
 * Read one file object of the sequence,<br>
 * than call QftFile::m_init( const QString &name ) to step to the next file object of the sequence,<br>
 * continue till all file objects have been read or the desired block size (s*l) has been reached.
 *
 * \param[in] v: used for file reading with reinterpret_cast
 * \param[in] s: read block having size s*l
 * \param[in] l: read block having size s*l
 * \return qint64: Size of read data
 */
qint64 QftFile::read( void *v, qint64 s, qint64 l ){
    qint64 size = 0;
    qint64 rest = s * l;

//    qDebug() << rest << m_rest;

    while( rest > m_rest ){
//        if( !m_rest ){
//            qDebug() << __FUNCTION__;
//            continue;
//        }
        if( m_rest )
            size += m_file.read( reinterpret_cast< char *>(v), m_rest );

        rest -= m_rest;
        m_filePosition += m_rest;
        if( !m_step() ){
//            qDebug() << __FUNCTION__ << m_infoIndex;
            return size;
        }
    }

    if( !rest ) return size;
//    qDebug() << size;
    size += m_file.read( reinterpret_cast<char *>(v), rest );
//    qDebug() << m_file.fileName() << m_file.errorString();


    m_rest -= rest;
    m_filePosition += rest;
//    qDebug() << size;

    return size;
}

/*!
 * \brief Write file sequence
 *
 * Write a qft file object of maximal size or size of required block (s*l) if smaller,<br>
 * create next file to write,<br>
 * continue till the required block size has been written.
 *
 * \return void
 */
void QftFile::write( void *v, qint64 s, qint64 l ){

    qint64 rest = s * l;

    if( !m_rest && rest ) m_step();

    while( rest > m_rest ){
        m_file.write( reinterpret_cast< const char *>(v), m_rest );

        rest -= m_rest;
        m_filePosition += m_rest;
        if( !m_step() ) return;
    }

    if( !rest ) return;

    m_file.write( reinterpret_cast< const char *>(v), rest );
    m_rest -= rest;
    m_filePosition += rest;
}

/*!
 * \brief Return the size of the file sequence.
 *
 * \return Size of the file sequence of 0 if sequence is valid.
 */
qint64 QftFile::size(){

    if( m_infoVector.isEmpty() ) return 0;

    return m_infoVector.last().offset() +
           m_infoVector.last().size();
}

/*! \brief Return information about a qft file object of the file sequence
 *
 * m_infoVector is used to handle a set of qft files,<br>
 * a QftFileInfo object about a file of the sequence can be obtained using this function.
 *
 * \param[in] i Position in the QVector
 * \return QftFileInfo object at position i
 */
QftFileInfo *QftFile::fileInfo( const int i ){

    if( i < 0 || i >= m_infoVector.size() )
        return 0;

    return &m_infoVector[ i ];
}

/*! \brief Set the name of the current qft file sequence
 *
 * Change or set the name of a qft file sequence.<br>
 * If a qft file sequence is currently opened the file sequence get closed before.
 *
 * \param[in] name Name of the qft file or a directory which contain qft files
 * \return void
 */
void QftFile::setFileName( const QString &name ){

    close();
    m_init( name );
}


/*! \brief Initialize m_infoVector with all files of a file sequence
 *
 * A file sequence can be one qft file or all qft files of a directory
 * or all files which match a filename pattern.
 *
 * \see For description of file patterns QftFile::m_fileFilter( QString &base, int &l )
 *
 * \param[in] name Name of the qft file or a qft file directory
 * \return void
 */
void QftFile::m_init( const QString &name ){

    m_fileInfo = QFileInfo( name );
    m_fileName = m_fileInfo.filePath();
    m_baseName = m_fileInfo.fileName();

    m_infoVector.clear();

    QRegExp rx( m_fileFilter( m_baseName, m_indexLength ));

    if( m_indexLength == 0 ){
        m_infoVector.append( QftFileInfo( name ));
        return;
    }

    QStringList files = m_fileInfo.dir().entryList();
    foreach( QString n, files ){

        if( !rx.exactMatch( n )) continue;

        n.prepend( m_fileInfo.path() + "/" );
//        qDebug() << n;
        m_infoVector.append( QftFileInfo( n, 0, rx.cap( 1 ).toInt() ));
    }

    qSort( m_infoVector );

    qint64 o = 0;
    for( int i=0; i < m_infoVector.size(); ++i ){
        m_infoVector[ i ].setOffset( o );
        o +=  m_infoVector[ i ].size();
    }
}

/*! \brief Set a filter for qft files in a specified directory
 *
 * Elements of a file sequence can be one file, all files in a directory or files which match a given
 * file pattern:<br>
 * - file.txt		<br>one single file object
 * - file[3].txt	<br>files which contain "file" + three digits + ".txt" belongs to the file sequence:<br>
 *			- file001.txt
 *			- file002.txt
 *			- ...<br>
 *
 * - file[].txt		<br>files which contain "file" + a diggit + ".txt" belongs to the file sequence:<br>
 *			- file1.txt
 *			- file2.txt
 *			- ...<br>
 *
 * \todo Describe the pattern of filename filters.
 *
 * \param[in] base Base file name or directory
 * \param[out] l Index Length
 * \return QString filtered file name
 */
QString QftFile::m_fileFilter( QString &base, int &l ){

    QRegExp rx( "^(.*)\\[(\\d*)\\](.*)$" );
    l = 0;

    if( !rx.exactMatch( base ))
        return base.prepend( m_fileInfo.path() + "/" );

    QString qs = rx.cap( 2 );
    if( qs.isEmpty() ){
        l  = -1;
        qs = "(0|(?:[1-9]\\d*))";
    }else{
        l  = qs.toInt();
        qs = "(\\d{" + qs + "})";
    }

    base = rx.cap( 1 ) + "%1" + rx.cap( 3 );
//    base = QRegExp::escape( rx.cap( 1 )) + "%1" +
//           QRegExp::escape( rx.cap( 3 ));
    base.prepend( m_fileInfo.path() + "/" );

    return QString( "^%1%2%3$" ).arg( QRegExp::escape( rx.cap( 1 )))
                                .arg( qs )
                                .arg( QRegExp::escape( rx.cap( 3 )));
}

/*! \brief Step through the files of m_infoVector and open the next file as m_file
 *
 * In ReadOnly mode increment m_infoIndex and set m_rest to size of the new QftFileInfo.<br>
 * A new file for writing can always be opened.
 *
 * \return bool true if next file has been set.<br>
 *		false in read only mode if the last m_infoVector element has been reached.
 */
bool QftFile::m_step(){

    if( m_file.openMode() != QIODevice::NotOpen ){
        m_infoVector[ m_infoIndex ].refresh();
        m_file.close();
    }
    emit sigFinished( m_infoIndex );

    ++m_infoIndex;

    if( m_infoIndex >= m_infoVector.size() ){
        // make difference between reading and writing

        if( m_openMode & QIODevice::WriteOnly ){
            int     index = m_infoVector.last().index() + 1;
            QString name = m_baseName.arg( index, m_indexLength, 10, QChar( '0' ));
            m_infoVector.append( QftFileInfo( name, m_filePosition, index ));
            m_rest = dataSizeLimit;

        }else
            return false;
    }else{
        if( m_openMode & QIODevice::ReadOnly ){
            m_rest = m_infoVector[ m_infoIndex ].size();
            emit sigStarted( m_infoIndex );
        }
    }
    m_file.setFileName( m_infoVector[ m_infoIndex ].filePath() );
    m_file.open( m_openMode );
    return true;
}








//****************************************************************************
//                    UNUSED  REST
//****************************************************************************



#ifdef OLD



//****************************************************************************
//* m_init (private)
//****************************************************************************


QString QftFile::fileName( void ){

    return fileName( -1 );
}

QString QftFile::fileName( int idx ){

    return m_baseName.arg( idx, m_indexLen, 10, QChar( '0' ));
};



//    if( idx < 0 || idx >= m_infoVector.size() )
//        return fileName( m_infoVector[ m_info ] );
//
//    return fileName( m_infoVector[ idx ] );
//}

QString QftFile::fileName( QftFileInfo &info ){

    return info.nam;

//
//    if( !m_indexLen )
//        return m_baseName;
//
//    return m_baseName.arg( info.idx, m_indexLen, 10, QChar( '0' ));
}

QString QftFile::fileName( const QString &name ){

    return name.section( '/', -1 );
}


QString QftFile::path( QString name ){

    name = QDir::cleanPath( name );
    name.chop( QftFile::fileName( name ).size() );

    return name;
}

QString QftFile::filter( const QString &name, int &l ){

    QRegExp rx( "\\[(\\d*)\\]" );
    QString n( fileName( name ) );
    l = 0;

    if( rx.indexIn( n ) == -1 ) return n;

    QString rs = rx.cap( 1 );
    if( rs.isEmpty() ){
        l  = -1;
        rs = "(0|(?:[1-9]\\d*))";
    }else{
        l  = rs.toInt();
        rs = "(\\d{" + rs + "})";
    }

    return QRegExp::escape( n.section( rx, 0, 0 ) ) + rs +
           QRegExp::escape( n.section( rx, 1, 1 ) );
}

void QftFile::m_init( const QString &name ){

    m_path     = path( name );
    m_fileName =
    m_baseName = fileName( name );

    QRegExp rx( "\\[(\\d*)\\]" );
    int p = m_baseName.indexOf( rx );
    m_baseName.replace( p, rx.matchedLength(), "%1" );
    m_baseName.prepend( m_path );

    m_filter = QRegExp( QString( "^%1$" ).arg( filter( m_fileName, m_indexLen )));

    QDir dir( m_path );
    QStringList files = dir.entryList().filter( m_filter );

    qint64 ql;
    qint64 qp = 0;
    int    qi;

    foreach( QString qs, files ){
        m_filter.indexIn( qs );
        qi = m_filter.cap( 1 ).toInt();
        ql = QFileInfo( dir, qs ).size();
        m_infoVector << QftFileInfo( qs, 0, ql, qi );
    }

    qSort( m_infoVector );
    foreach( QftFileInfo qi, m_infoVector ){
        qi.pos = qp;
        qp += qi.len;
    }
}



int QftFile::m_step(){

    m_file.close();

    emit finished( m_info );

    m_info++;
    if( m_info >= m_infoVector.size() ){
        // make difference between reading and writing

        if( m_openMode & QIODevice::WriteOnly ){
            m_infoVector.append( QftFileInfo( fileName( m_info ), m_fpos, 0, m_info ) );
            m_rest = DataSizeLimit;

        }else
            return -1;
    }else{
        if( m_openMode & QIODevice::ReadOnly ){
            m_rest = m_infoVector[ m_info ].len;
            emit started( m_info );
        }
    }

    m_file.setFileName( fileName( m_info ) );
    m_file.open( m_openMode );

    return 0;
}



void QftFile::write( const void *v, qint64 s, qint64 l ){

    qint64 rest = s * l;

    if( !m_rest && rest ) m_step();

    while( rest > m_rest ){
        m_file.write( reinterpret_cast< const char *>(v), m_rest );
        rest   -= m_rest;
        m_fpos += m_rest;
        if( m_step() ) return;
    }

    if( !rest ) return;

    m_file.write( reinterpret_cast< const char *>(v), rest );
    m_rest -= rest;
    m_fpos += rest;
}









#ifdef OLD

//****************************************************************************
//* qfopen
//! The qfopen function opens the file whose name must be set in the
//! constructor. The argument mode points to a string beginning with one of
//! the following sequences (additional characters may follow these sequences,
//! but will be ignored). This functions behaves exactly like the fopen
//! function.
//! \param[in] mode openmode, possible values:
//! \arg \c "r": Open file for reading (\c QIODevice::ReadOnly). The
//!             filepointer points to the beginning of the file.
//! \arg \c "r+": Open file for reading and writing
//!             (\c QIODevice::ReadWrite). The filepointer points to the
//!             beginning of the file.
//! \arg \c "w": Truncate file to zero length or create file for writing
//!             (\c QIODevice::WriteOnly). The filepointer points to
//!        	    the beginning of the file.
//! \arg \c "w+":Truncate file to zero length or create file for reading
//!             and writing (\c QIODevice::ReadWrite). The filepointer
//!			    points to the beginning of the file.
//! \arg \c "a+" and \c "a":
//!       	    Open for appending (writing at end of file \c QIODevice::Append)
//!             The file is created if it does not exist. The
//!             filepointer points to the end of the file.
//! \param[in] idx index (default=-1)
bool QftFile::open( const char *mode, int idx ){

    QTextStream cerr(stderr);

//    cerr << __PRETTY_FUNCTION__ << endl;


    if( m_base.isEmpty() ) return false;

    cerr << "filename exists: ===========>" << m_base << endl;



    if( m_file.openMode() != QIODevice::NotOpen ){
        qWarning("QftFile::open: File already open");
        return false;
    }

    switch( *mode++ ){

        case 'w':
            foreach( QftFileInfo *info, m_infoVector )
                QFile::remove( fileName( info->idx ) );
            m_infoVector.clear();
            m_infoVector.append( new QftFileInfo( 0, 0, idx ) );
            m_info = m_infoVector.first();
            m_fpos = 0;
            m_mode = ( *mode == '+' ) ? QIODevice::ReadWrite :
                                        QIODevice::WriteOnly;
            break;

        case 'r':
            m_info = m_infoVector.first();
            m_fpos = 0;
            m_mode = ( *mode == '+' ) ? QIODevice::ReadWrite :
                                        QIODevice::ReadOnly;
            break;

        case 'a':
            if( *mode == '+' ){
                m_info = m_infoVector.first();
                m_fpos = 0;
                m_mode = QIODevice::Append | QIODevice::ReadOnly;
            }else{
                m_info = m_infoVector.last();
                m_fpos = m_info->pos + m_info->len;
                m_mode = QIODevice::Append;
            }
            break;

        default:
            return false;
    }

    m_file.setFileName( fileName( idx ) );

    cerr << "File: " << m_file.fileName() << endl;

    if( !m_file.open( m_mode ) ){
        qWarning("QftFile::open: File cannot be opened");
        return false;
    }

    return true;
}

//****************************************************************************
//* qfopen
//! The qfopen function opens the file whose name is defined by the QString
//! name. The argument mode points to a string beginning with one of
//! the following sequences (additional characters may follow these sequences,
//! but will be ignored).
//! \param[in] name name of the file (can be indexed)
//! \param[in] mode openmode, possible values:
//! \arg \c "r": Open file for reading (\c QIODevice::ReadOnly). The
//!             filepointer points to the beginning of the file.
//! \arg \c "r+": Open file for reading and writing
//!             (\c QIODevice::ReadWrite). The filepointer points to the
//!             beginning of the file.
//! \arg \c "w": Truncate file to zero length or create file for writing
//!             (\c QIODevice::WriteOnly). The filepointer points to
//!        	    the beginning of the file.
//! \arg \c "w+":Truncate file to zero length or create file for reading
//!             and writing (\c QIODevice::ReadWrite). The filepointer
//!			    points to the beginning of the file.
//! \arg \c "a+" and \c "a":
//!       	    Open for appending (writing at end of file \c QIODevice::Append)
//!             The file is created if it does not exist. The
//!             filepointer points to the end of the file.
//! \param[in] idx index (default=-1)
bool QftFile::open( const QString &name, const char *mode, int idx ){

    QftFile::init( name );

    return open( mode, idx );
}

//****************************************************************************
//* qfopen
//! The qfopen function opens the file whose name must be set in the
//! constructor. The argument mode is an enumerated value defined by QIODevice.
//! \param[in] mode openmode, possible values:
//! \arg \c QIODevice::ReadOnly: Open file for reading ("r"). The
//!             filepointer points to the beginning of the file.
//! \arg \c QIODevice::ReadWrite: Open file for reading and writing
//!             ("r+"). The filepointer points to the
//!             beginning of the file.
//! \arg \c QIODevice::WriteOnly: Truncate file to zero length or create file for writing
//!             ("w"). The filepointer points to
//!        	    the beginning of the file.
//! \arg \c QIODevice::Append:
//!       	    Open for appending (writing at end of file, "a" or "a+")
//!             The file is created if it does not exist. The
//!             filepointer points to the end of the file.
//! \param[in] idx index (default=-1)
bool QftFile::open( const QIODevice::OpenMode mode, int idx ){

    if( mode & QIODevice::Append )
        return open( ( mode & QIODevice::ReadOnly ) ? "a+" : "a", idx );
    else if( mode & QIODevice::WriteOnly )
        return open( ( mode & QIODevice::ReadOnly ) ? "r+" : "w", idx );
    else if( mode & QIODevice::ReadOnly )
        return open( "r", idx );

    return false;
}

//****************************************************************************
//* qfopen
//! The qfopen function opens the file whose name is defined by the QString
//! name. The argument mode is an enumerated value defined by QIODevice.
//! \param[in] name name of the file (can be indexed)
//! \param[in] mode openmode, possible values:
//! \arg \c QIODevice::ReadOnly: Open file for reading ("r"). The
//!             filepointer points to the beginning of the file.
//! \arg \c QIODevice::ReadWrite: Open file for reading and writing
//!             ("r+"). The filepointer points to the
//!             beginning of the file.
//! \arg \c QIODevice::WriteOnly: Truncate file to zero length or create file for writing
//!             ("w"). The filepointer points to
//!        	    the beginning of the file.
//! \arg \c QIODevice::Append):
//!       	    Open for appending (writing at end of file, "a" or "a+")
//!             The file is created if it does not exist. The
//!             filepointer points to the end of the file.
//! \param[in] idx index (default=-1)
bool QftFile::open( const QString &name,
                      const QIODevice::OpenMode mode, int idx ){
    QftFile::init( name );

    return open( mode, idx );
}

//****************************************************************************
//* qfclose
bool QftFile::close(){
    if( m_file.openMode() == QIODevice::NotOpen ) return true;

    m_file.close();
    return true;
}

//****************************************************************************
//* qfseek
//! \arg \c QFT_SEEK_CUR
//! \arg \c QFT_SEEK_END
//! \arg \c QFT_SEEK_SET
void QftFile::seek( qint64 pos, int mode ){

    if( m_mode == QIODevice::Append ) return;

    switch( mode ){
        case QFT_SEEK_CUR:
            m_fpos += pos;
            break;
        case QFT_SEEK_END:
            m_fpos = abs( m_info->pos + m_info->len + pos );
            break;
        case QFT_SEEK_SET:
            m_fpos = abs( pos );
        default:
            break;
    }
}



//****************************************************************************
//*
/*
void QftFile::operator<<( const quint8 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const quint16 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const quint32 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const quint64 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const qint8 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const qint16 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const qint32 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const qint64 v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const bool v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const float v ){
    QftFile::pwrite( v ); }
void QftFile::operator<<( const double v ){
    QftFile::pwrite( v ); }
*/
//****************************************************************************
/*
void QftFile::read( void *v, qint64 s, qint64 l ){

    read( reinterpret_cast<quint8 *>(v), s*l );
}
*/
//****************************************************************************
/*
void QftFile::operator>>( quint8 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( quint16 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( quint32 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( quint64 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( qint8 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( qint16 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( qint32 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( qint64 &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( bool &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( float &v ){
    QftFile::pread( v ); }
void QftFile::operator>>( double &v ){
    QftFile::pread( v ); }
*/

//****************************************************************************
//* STATIC FUNCTIONS
//****************************************************************************

//****************************************************************************
//*

//****************************************************************************
//*
QString QftFile::path( QString name ){

    name = QDir::cleanPath( name );
    name.chop( QftFile::fileName( name ).size() );

    return name;
}

//****************************************************************************
//*
void QftFile::remove( const QString &name ){

    int l;

    QDir dir( path( name ) );
    QStringList fl = dir.entryList().filter( QRegExp( filter( name, l ) ) );

    foreach( QString f, fl ) QFile::remove( path( name ) + f );
}

//****************************************************************************
//*
void QftFile::rename( const QString &oname,
                       const QString &nname, bool renumber ){
}

//****************************************************************************
//*
QString QftFile::filter( const QString &name, int &l ){

    QRegExp rx( "\\[(\\d*)\\]" );
    QString n( fileName( name ) );
    l = 0;

    if( rx.indexIn( n ) == -1 ) return n;

    QString rs = rx.cap( 1 );
    if( rs.isEmpty() ){
        l  = -1;
        rs = "(\\d+)";
    }else{
        l  = rs.toInt();
        rs = "(\\d{" + rs + "})";
    }

    return QRegExp::escape( n.section( rx, 0, 0 ) ) + rs +
           QRegExp::escape( n.section( rx, 1, 1 ) );
}


//****************************************************************************
//* PRIVATE FUNCTIONS
//****************************************************************************


//****************************************************************************
//* seek
void QftFile::pseek(){

    if( ( m_mode & QIODevice::Append ) )
        m_fpos = m_infoVector.last()->pos + m_infoVector.last()->len;

    if( m_info->pos + m_file.pos() == m_fpos ) return;

    if( m_infoVector.last()->pos + m_infoVector.last()->len <= m_fpos ){

        foreach( QftFileInfo *it, m_infoVector )
            if( m_fpos >= it->pos && m_fpos <= it->pos + it->len ){

                if( m_info->idx != it->idx ){
                    m_info = it;
                    m_file.close();
                    m_file.setFileName( fileName( m_info->idx ) );
                    m_file.open( m_mode );
                    m_file.seek( m_fpos - m_info->pos );
                }
                break;
            }
    }else{

        if( m_info->idx != m_infoVector.last()->idx ){
            m_info = m_infoVector.last();
            m_file.close();
            m_file.setFileName( fileName( m_info->idx ) );
            m_file.open( m_mode );
            m_file.seek( m_info->len );
        }

        while( m_info->len + m_info->pos < m_fpos ){
            if( m_info->len >= DataSizeLimit ){
                m_infoVector.append(
                    new QftFileInfo( m_info->pos + m_info->len, 0, m_info->idx + 1 ) );
                m_info = m_infoVector.last();
                m_file.close();
                m_file.setFileName( fileName( m_info->idx ) );
                m_file.open( m_mode );
            }

            m_data << static_cast<char>( 0 );
        }
    }
}

#endif
#endif

