/*!
 * \file qftFile.qt.hpp
 * \brief This file contains the class defintions of QftFile and QftFileInfo which helps to handle file sequences
 *
 * \author  Dipl. Inf. Manfred Reimann
 *
 * $Revision$
 *
 * <dl class="rcs">
 *    <dt><b>Product Name:</b></dt><dd>FTGS Dacff</dd>
 * </dl>
 *
 * $Date$
 *
 * <dl class="rcs">
 *    <dt><b>Classification:</b></dt><dd>UNCLASSIFIED</dd>
 * </dl>
 *
 * <dl class="rcs">
 *    <dt><b>Copyright:</b></dt><dd>&copy; EADS 2010</dd>
 * </dl>
 *
 * <dl class="rcs">
 *    <dt><b>Description:</b></dt><dd>File Handling of qft files (sequences of files)</dd>
 * </dl>
 *
 * \todo documentation missing and version number
 *
 *************************************************************************
 * <dl class="rcs"><dt><b>History:</b></dt><dd>
 * <pre>
 * $Log$
 * </pre>
 * 12.03.07 MR  initiation<br>
 * </dd></dl>
 *************************************************************************
*/


#ifndef _QFTFILE_QT_HPP_
#define _QFTFILE_QT_HPP_

#include "QDebug"
#include "QFile"
#include "QFileInfo"
#include "QVector"

/*!
 * \brief The class QftFileInfo handle information about a single qft file object
 *
 * QftFileInfo contains the following information about qft files:<br>
 * File offset and index <br>
 *
 * The handling of offset and index can be done with this class.<br>
 *
 * \todo expand the class description
 */
class QftFileInfo : public QFileInfo {

    public:
        /*! \brief Default constructor
	*/
        QftFileInfo(){}

	/*!
	* \brief Alternative constructor
	*
	* Initialise the qft file info object:<br>
	* Set the filename, offset and index.
	*
	* \param[in] n Name of the qft file object
	* \param[in] o Initial file offset
	* \param[in]  i Initial file index
	*/
        QftFileInfo( const QString &n, qint64 o=0, int i=0 ) :
            QFileInfo( n ),
            m_offset( o ),
            m_index( i ){}

	/*!
	* \brief Returns the current file index
	*
	* \return Current file index
	*/
        int    index(){ return m_index; }

	/*!
	* \brief Returns the current file offset
	*
	* \return Current file offset
	*/
        qint64 offset(){ return m_offset; }

	/*!
	* \brief Sets the current file offset
	*
	* \param[in] v New file offset
	*/
        void   setOffset( qint64 v ){ m_offset = v; }

	/*!
	* \brief Less than operator for qft files
	*
	* Compares the file index of the qft file object represented with this class with the index of a file info
	* object given as parameter. If the current index is smaller return true.
	*
	* \param[in] v Reference to a qft file object
	* \return       true if index of the qft file is smaller as the refernce class index (v.m_index)
	*/
        bool   operator<( const QftFileInfo &v ) const {
            return m_index < v.m_index;
        }

    private:
        qint64 m_offset; 			//!< qft file offset (size of file sequence files which contain data which belongs before this data)
        int    m_index;				//!< qft file index (id which indicate the position of the data relativ to other qft sequence files)
};

/*!
 * \brief The class QftFile handle sequences of files (read/write process)
 *
 * Qft (<b>Q</b>t for <b>f</b>light <b>t</b>est ) -Files can contain various data.<br>
 * Qft files can be fragmented in different single files (file sequences) to handle large file sizes.<br>
 * This file sequence can be accessed like one file with the help of this class (they behave like one file).<br>
 * One qft file has a maximal size of 1 GB.<br>
 *
 * \todo review the class description
 */
class QftFile : public QObject {

    Q_OBJECT

    public:
        QftFile();
        QftFile( const QString &name );
        ~QftFile();

        bool         open( const char *mode );
        bool         open( QIODevice::OpenMode mode );
        void         close();


        bool         setPermissions( QFile::Permissions permissions );


        template <typename T>
        qint64       read( T *v, qint64 l );
        qint64       read( void *v, qint64 s, qint64 l );

        template <typename T>
        void         write( T *v, qint64 l );
        void         write( void *v, qint64 s, qint64 l );

        qint64       size();
//        bool         flush() { return m_file.waitForBytesWritten( -1 ); }
        QftFileInfo *fileInfo( const int i );

        void         setFileName( const QString &name );

        qint64       dataSizeLimit;		//!< Maximal size of a single qft file

    signals:
        void         sigStarted( int index );  	/*!< \brief Start to read/write a qft file
						 *
						 * (in write mode only for the first file of the sequence)
						 * \param[in] index id in m_infoVector
						 */

        void         sigFinished( int index ); 	/*!< \brief Finished to read/write a file
						 * \param[in] index id in m_infoVector
						 */

    private:
        void         m_init( const QString & );
        bool         m_step();
        QString      m_fileFilter( QString &base, int &l );

	// information about the current sequence file object
        qint64       m_rest;			//!< Size of unreaded/free bytes of the current sequence file object
        qint64       m_filePosition;		//!< Current read/write position for the current qft file sequence object
        qint64       m_fileSize;		//!< Size of the current single qft file sequence object
	QIODevice::OpenMode     m_openMode; 	//!< Current open mode for the current qft file sequence object (not opened, write or read  mode)
	QFile        m_file;			//!< Handle to the current sequence file object
        QFileInfo    m_fileInfo;		//!< File Information about the currently opened qft file sequence object

	// file sequence information
        int          m_indexLength;		//!< Length of file names. If length equals 0 only one sequence object can be used.
	QString      m_fileName;		//!< Path to the qft files sequence object
        QString      m_baseName;		//!< File base name, needed to handle file sequences

        int                     m_infoIndex;	//!< Index in m_infoVector of currently active qft file sequence object
        QVector<QftFileInfo>    m_infoVector;	//!< Helps to handle file sequences, Vector of QftFileInfo objects
};

/*! \brief Function template to read file sequences
 *
 *
 * \param[in] v used for file reading with reinterpret_cast
 * \param[in] l read block having size sizeof(T)*l
 */
template <typename T>
qint64 QftFile::read( T *v, qint64 l ){
    return read( reinterpret_cast< void *>(v), sizeof( T ), l ) / sizeof( T );
}

/*! \brief Function template to write file sequences
 *
 *
 * \param[in] v used for file reading with reinterpret_cast
 * \param[in] l write block having size sizeof(T)*l
 */
template <typename T>
void QftFile::write( T *v, qint64 l ){
    write( reinterpret_cast< void *>(v), sizeof( T ), l );
}




//****************************************************************************
//                    UNUSED  REST
//****************************************************************************



#ifdef OLD_VERSION

class QftFile : public QObject {

    Q_OBJECT

    struct QftFileInfo {
        QftFileInfo(){}
        QftFileInfo( QString name, qint64 p=0, qint64 l=0, int i=-1 ) :
            nam( name ),
            pos( p ),
            len( l ),
            idx( i ){}
        QString   nam;
        qint64    pos;
        qint64    len;
        int       idx;
        bool      operator<( const QftFileInfo &v ) const { return idx < v.idx; }
    };



//    typedef QMap<int, TInfo> TInfoVector;

//    QMap<int, QftFileInfo>::iterator m_i;


//    typedef QVector<QftFileInfo> TInfoVector;
//    typedef QIODevice::OpenMode TMode;

    public:

        QftFile();
        QftFile( const QString &name );

        ~QftFile();

        void            setFileName( const QString & );

        int             open( const char *mode );
        int             open( QIODevice::OpenMode mode );












        template <typename T>
        qint64          read( T *v, qint64 l );

        template <typename T>
        void        	write( const T *v, qint64 l );
        void        	write( const void *v, qint64 s, qint64 l );

        QString         fileName( void );
        QString         fileName( int idx=-1 );
        QString         fileName( QftFileInfo &info );
        static QString  fileName( const QString & );




        static QString  path( QString );    				// path includig the trailing /
        static QString  filter( const QString &, int & );
//        int             open( const QString &, const char *, int idx=-1 );
        void            close();

        static const qint64 DataSizeLimit;


    signals:
        void     finished( int idx );
        void     started( int idx );


    private:

        void                    m_init( const QString & );
        int                     m_step();

        QIODevice::OpenMode     m_openMode;
        QVector<QftFileInfo>    m_infoVector;
        int                     m_infoIndex;






//        void            m_info();

        QString         m_baseName;
        QString         m_fileName;
        // QftFILE
        QRegExp         m_filter;
//        QString         m_base;
        QFile           m_file;
//        TMode		    m_mode;
        int             m_info;
        QString         m_path;
//        TInfoVector     m_infoVector;
        // INDEX:
//        int        	    m_indexMin;
//        int             m_indexMax;
        int             m_indexLen;
        // FILE:
        qint64          m_rest;
        qint64          m_fpos; // current position in current file
        qint64          m_flen; // current length of current file




//        int             m_length;
/*
        //        template <typename T>
//        quint64 read( T *v, quint64 l );
#ifdef OLD
        enum QftFileSeekMode {
            QFT_SEEK_SET,
            QFT_SEEK_CUR,
            QFT_SEEK_END
        };

        bool        	open( const char *mode, int idx=-1 );
        bool        	open( const QString &name, const char *mode, int idx=-1 );
        bool        	open( const QIODevice::OpenMode mode, int idx=-1 );
        bool        	open( const QString &name,
                              const QIODevice::OpenMode mode, int idx=-1 );
        TMode           openMode(){ return m_mode; }
//      bool        	reopen();
        bool        	close();

        template <typename T>
        void        	read( T *v, qint64 l );
        void        	read( void *v, qint64 s, qint64 l );

        template <typename T>
        void        	write( const T *v, qint64 l );
        void        	write( const void *v, qint64 s, qint64 l );

        void   	    	operator<<( const quint8 v );
        void        	operator<<( const quint16 v );
        void        	operator<<( const quint32 v );
        void        	operator<<( const quint64 v );
        void        	operator<<( const qint8 v );
        void        	operator<<( const qint16 v );
        void        	operator<<( const qint32 v );
        void        	operator<<( const qint64 v );
        void        	operator<<( const bool v );
        void        	operator<<( const float v );
        void        	operator<<( const double v );
        template <typename T>
        void        	operator<<( T &v );


        void   	    	operator>>( quint8 &v );
        void        	operator>>( quint16 &v );
        void        	operator>>( quint32 &v );
        void        	operator>>( quint64 &v );
        void        	operator>>( qint8 &v );
        void        	operator>>( qint16 &v );
        void        	operator>>( qint32 &v );
        void        	operator>>( qint64 &v );
        void        	operator>>( bool &v );
        void        	operator>>( float &v );
        void        	operator>>( double &v );
        template <typename T>
        void        	operator>>( T &v );

        qint64      	tell(){ return m_fpos; }
        qint64      	getpos(){ return tell(); }
        void        	setpos( qint64 v ){ seek( v ); }
        void        	seek( qint64, int mode=QFT_SEEK_SET );
        void        	rewind(){ seek( 0 ); }

        QString         fileName( void );
        QString         fileName( int idx );
        static QString  fileName( const QString & );
        static QString  path( QString );

        static void     chmod( QString & );
        static void     remove( const QString & );
        static void     rename( const QString &, const QString &, bool );
        static QString  filter( const QString &, int & );

        void            setByteOrder( const QDataStream::ByteOrder v ){ m_data.setByteOrder( v ); }

    signals:

        void    sigIndexIncreased( int );

    private:

        bool        	init( const QString &name );
        void        	step();
        void        	pseek();

        template <typename T>
        void        	pwrite( const T v );
        template <typename T>
        void        	pread( T &v );


        TInfoVector	    m_infoVector;
        TInfo          *m_info;
//        TInfo          &m_info;
        TMode			m_mode;

        qint64          m_fpos;

        QRegExp         m_filter;
        int             m_length;

#endif
        */
};



template <typename T>
void QftFile::write( const T *v, qint64 l ){
    write( reinterpret_cast< void *>(v), sizeof( T ), l );
}


template <typename T>
qint64 QftFile::read( T *v, qint64 l ){
    // later: big endian / little endian
    qint64 len;
    qint64 rest = sizeof( T ) * l;  // size in bytes
//    QTextStream cerr(stderr);
//    cerr << m_rest << endl;
    while( rest > m_rest ){

        rest -= m_rest;
        len = m_file.read( reinterpret_cast<char *>(v), m_rest );
        m_fpos += m_rest;
        if( m_step() ) return 0;
    }

    if( rest <= 0 ) return 0;

    len = m_file.read( reinterpret_cast<char *>(v), rest );
    m_fpos += rest;
    m_rest -= rest;

    return len / sizeof( T );
}









#ifdef OLD
//****************************************************************************
//*
template <typename T>
void QftFile::operator<<( T &v ){
    if( !sizeof( T ) ) return;

    seek();
    if( m_info.len + sizeof( T ) > DataSizeLimit && m_length ) step();

    m_fpos += sizeof( T );
    int size = ( m_info.pos + m_info.len ) - m_fpos;
    if( size < 0 )
        m_info.len -= size;

//    qfwrite();

}

//****************************************************************************
//*
template <typename T>
void QftFile::write( const T *v, qint64 l ){

    for( qint64 i = 0; i < l; ++i )
//        *this << v[i];
        m_data << v[i];
}

//****************************************************************************
//* private write
template<typename T>
void QftFile::pwrite( const T v ){

    if( !sizeof( T ) ) return;

    pseek();
    if( m_info->len + sizeof( T ) > DataSizeLimit && m_length ) step();

    m_fpos += sizeof( T );
    int size = ( m_info->pos + m_info->len ) - m_fpos;
    if( size < 0 )
        m_info->len -= size;
    m_data << v;
}

//****************************************************************************
//*

//****************************************************************************
//* private read
template<typename T>
void QftFile::pread( T &v ){

    if( !sizeof( T ) ) return;

    QTextStream cerr(stderr);

//    foreach( TInfo *info, m_infoVector ){
//        cerr << info->len << ", ";
//        cerr << info->pos << ", ";
//        cerr << info->idx << endl;
//    }
//    cerr << m_fpos << endl;
//    cerr << "^^^^^^^^^^^^^^^^^^^^^^^^" << endl;
    pseek();

    if( m_fpos + sizeof( T ) > m_info->len ) step();


//    if( m_info->len + sizeof( T ) > DataSizeLimit && m_length ) step();

    m_fpos += sizeof( T );
    int size = ( m_info->pos + m_info->len ) - m_fpos;
    if( size < 0 )
        m_info->len -= size;
    m_data >> v;
}
//****************************************************************************
#endif
#endif
#endif
